##############################################################
#
#   Humerus - Example game, with level editor
#
#   A version of the pong example with a level editor.
#
##############################################################

#
#  This is so you can run the example from the source
#  directory without installing humerus.
#

import os, sys
d = os.path.dirname
sys.path.insert(1, d(d(d(os.path.abspath(sys.argv[0])))))

#
#  Parse command line options and initialize display
#  before doing anything else.
#

from humerus.options import Options

class PongOptions(Options):

	frame_rate_option = 20
	screen_size_option = (640, 480)

options = PongOptions(with_editor = True)

#
#  Now do the rest.
#

import random
from pygame import Rect
from albow.screen import Screen
from albow.controls import Label, AttrRef
from albow.fields import IntField
from albow.layout import Row
from humerus.hegame import HEGame
from humerus.heshell import HEShell
from humerus.main_commands import MainCommandScreen
from humerus.edit_commands import EditCommandScreen

class Level(object):

#	is_completed = False
#	ball_position = None
#	score = 0
	
	def __init__(self, bat_size):
		self.bat_size = bat_size
		self.score = 0
		self.ball_position = None
		self.is_completed = False
	
	def level_is_completed(self):
		result = self.score >= 3
		return result
	
	def begin_frame(self):
		p = self.ball_position
		if p:
			x, y = p
			vx, vy = self.ball_speed
			w, h = shell.size
			x += vx
			y += vy
			if x < 0:
				x = -x
				vx = -vx
			elif x > w:
				r = game.bat_rect
				if r and not (y >= r.top and y <= r.bottom):
					self.ball_position = None
					self.score -= 1
					return
				else:
					self.score += 1
					x = w - (x - w)
					vx = -vx
			if y < 0:
				y = -y
				vy = -vy
			elif y > h:
				y = h - (y - h)
				vy = -vy
			self.ball_position = x, y
			self.ball_speed = vx, vy

class PongGame(HEGame):

	save_file_suffix = ".epongam"
	level_file_suffix = ".eponglev"
	level_set_suffix = ".epongset"

	bat_rect = None
	
	def get_bat_size(self):
		return self.level.bat_size
	
	def set_bat_size(self, x):
		self.level.bat_size = x
	
	bat_size = property(get_bat_size, set_bat_size)
	
	def create_level(self):
		return Level(bat_size = 30)

class PongPlayScreen(MainCommandScreen):

	ball_color = (255, 255, 255)
	ball_size = (8, 8)
	bat_color = (255, 255, 255)
	bat_width = 8

	def draw(self, surf):
		level = game.playing_level
		r = self.ball_rect()
		if r:
			surf.fill(self.ball_color, r)
		r = game.bat_rect
		if r:
			surf.fill(self.bat_color, r)
		name = game.level_name.split(".")[0]
		s = "Level: %s     Score: %s" % (name, level.score)
		buf = self.font.render(s, True, self.fg_color)
		r = buf.get_rect()
		r.midbottom = self.width // 2, self.height
		surf.blit(buf, r)
	
	def ball_rect(self):
		p = game.playing_level.ball_position
		if p:
			r = Rect((0, 0), self.ball_size)
			r.center = p
			return r
	
	def begin_frame(self):
		e = self.get_mouse()
		x, y = e.local
		r = Rect(0, 0, self.bat_width, game.playing_level.bat_size)
		r.midright = self.width, y
		game.bat_rect = r
		self.invalidate()
	
	def mouse_down(self, e):
		level = game.playing_level
		if level.ball_position is None:
			r = game.bat_rect
			if r:
				level.ball_position = r.midleft
				vx = -10
				vy = random.randint(-10, 10)
				level.ball_speed = vx, vy

class PongEditScreen(EditCommandScreen):

	def __init__(self, shell):
		EditCommandScreen.__init__(self, shell)
		lbl = Label("Bat Size")
		fld = IntField(width = "999", min = 1, ref = AttrRef(game, 'bat_size'))
		contents = Row([lbl, fld])
		self.add_centered(contents)
		fld.focus()
		
class PongShell(HEShell):

	game_title = "Python Pong"

	def __init__(self, game, options):
		HEShell.__init__(self, game, options)
		self.main_menu_screen = MainCommandScreen(self, with_menu = True)
		self.edit_menu_screen = EditCommandScreen(self, with_menu = True)
		self.play_screen = PongPlayScreen(self)
		self.editor_screen = PongEditScreen(self)


game = PongGame()
shell = PongShell(game, options)
shell.run()
